#   Copyright (C) 1987-1999 by Jeffery P. Hansen
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Last edit by hansen on Mon May 17 11:06:38 1999

#
# File menu
#
proc tkg_makeFileMenu {m} {
    menu $m
    $m add command -label "New" -underline 0 -command tkg_new -accelerator "Ctl-X k"
    $m add command -label "Open..." -underline 0 -command tkg_load -accelerator "Ctl-X Ctl-F"
    $m add command -label "Open Library..." -underline 5 -command tkg_loadLibrary -accelerator "Ctl-X Ctl-L"
    $m add command -label "Save" -underline 0 -command tkg_saveCurrent -accelerator "Ctl-X Ctl-S"
    $m add command -label "Save..." -underline 1 -command tkg_save -accelerator "Ctl-X Ctl-W"
    $m add separator
    $m add command -label "Print..." -underline 0 -accelerator "Ctl-X Ctl-P" -command tkg_printDlg
    $m add command -label "Options..." -underline 2 -command tkg_editOptions -accelerator "Ctl-X o"
    $m add separator
    $m add command -label "Quit"  -underline 0 -command { tkg_exit } -accelerator "Ctl-X Ctl-C"
}

######################################################################
#
# Edit menu
#
proc tkg_makeEditMenu {m} {
    menu $m
    $m add command -label "Cut" -underline 0 -accelerator "Ctl-W" -command gat_cutToBuf
    $m add command -label "Copy" -underline 0 -accelerator "Ctl-X x" -command gat_copyToBuf
    $m add command -label "Paste" -underline 0 -accelerator "Ctl-Y" -command gat_yankFromBuf
    $m add command -label "Select All" -underline 0 -accelerator "Ctl-X a" -command gat_selectAll
    $m add command -label "Find..." -underline 0 -accelerator "Ctl-F" -command tkg_findObject
}

######################################################################
#
# Tool selection menu
#
proc tkg_makeToolMenu {m} {
    global tkg_allowDebug

    menu $m
    $m add radiobutton -label "Move/Connect" -underline 0 -variable mode -value 1 -accelerator F1
    $m add radiobutton -label "Delete Gate" -underline 0 -variable mode -value 3 -accelerator F2
    $m add radiobutton -label "Cut Wire" -underline 0 -variable mode -value 0 -accelerator F3
    $m add radiobutton -label "Invert" -underline 0 -variable mode -value 2 -accelerator F4
    $m add radiobutton -label "Bit Width" -underline 0 -variable mode -value 9 -accelerator F5
    if { $tkg_allowDebug } {
	$m add radiobutton -label "Debug" -underline 0 -variable mode -value 12 -accelerator F6
    }
    $m add separator
    $m add radiobutton -label "Rot 0" -underline 4 -variable rot -value 0 -accelerator Ctl-F1
    $m add radiobutton -label "Rot 90" -underline 4 -variable rot -value 90 -accelerator Ctl-F2
    $m add radiobutton -label "Rot 180" -underline 4 -variable rot -value 180 -accelerator Ctl-F3
    $m add radiobutton -label "Rot 270" -underline 4 -variable rot -value 270 -accelerator Ctl-F4
}

######################################################################
#
# Simulator menu
#
proc tkg_makeSimMenu {m s} {
    menu $m
    $m add command -label "Begin Simulation" -underline 0 -command gat_simulate -accelerator "Ctl-S b"
    $m add separator

    $m add command -label "Run" -underline 0 -command tkg_simRun -accelerator "Ctl-S g" -state $s
    $m add command -label "Pause" -underline 0 -command tkg_simStop  -accelerator "Ctl-S s" -state $s
    $m add command -label "Step Epoch(s)" -underline 0 -command tkg_simStep -accelerator "Space" -state $s
    $m add command -label "Step Cycle" -underline 5 -command tkg_simCycle -state $s -accelerator "Tab"
    $m add command -label "Breakpoint..." -underline 0 -state $s -command tkg_editBreakpoints -accelerator "Ctl-S k"
    $m add command -label "Exec. Script..." -underline 1 -state $s -command tkg_doSimScript -accelerator "Ctl-S x"
    $m add separator
    $m add command -label "Load Memory..." -underline 1 -state $s -command tkg_simLoadMem -accelerator "Ctl-S l"
    $m add command -label "Dump Memory..." -underline 1 -state $s -command tkg_simDumpMem -accelerator "Ctl-S d"
}

######################################################################
#
# Module Operations
#
proc tkg_makeModuleMenu {m} {
    menu $m
    $m add command -label "Open" -underline 0 -command gat_openBox -accelerator >
    $m add command -label "Close" -underline 0 -command gat_closeBox -accelerator <
    $m add separator
    $m add command -label "Set Interface" -underline 0 -command gat_setBlockDesc -accelerator "Ctl-b s"
    $m add command -label "Edit Interfaces..." -underline 0 -command gat_editBlockDesc -accelerator "Ctl-b e"
    $m add separator
    $m add command -label "New..." -underline 0 -accelerator "Ctl-b n" -command tkg_blockNew
    $m add command -label "Delete..." -underline 0 -accelerator "Ctl-b d" -command tkg_blockDelete
    $m add command -label "Copy..." -underline 0 -accelerator "Ctl-b c" -command tkg_blockCopy
    $m add command -label "Rename..." -underline 0 -accelerator "Ctl-b r" -command tkg_blockRename
    $m add separator
    $m add command -label "Circuit Properties..." -underline 0 -command tkg_editCircProp -accelerator "Ctl-x E"
}

######################################################################
#
# Gate menu
#
proc tkg_makeGateMenu {m} {
    menu $m
    $m add command -label "Add Input" -underline 4 -command gat_addIn -accelerator \[
    $m add command -label "Add Output" -underline 4  -command gat_addOut -accelerator \]
    $m add command -label "Add BiDir" -underline 6 -command gat_addInOut -accelerator -
    $m add command -label "Change Type" -underline 1  -command gat_changePinDir -accelerator @
    $m add separator
    $m add command -label "Anchor Selected" -underline 0 -command {gat_anchor 1} -accelerator "Ctl-x f"
    $m add command -label "Unanchor Selected" -underline 0 -command {gat_anchor 0} -accelerator "Ctl-x u"
    $m add separator
    $m add command -label "Properties..." -underline 0 -command gat_editProps -accelerator E
    $m add command -label "Replicate" -underline 0 -command gat_replicate -accelerator Ctl-V
    $m add command -label "Delete" -underline 0 -accelerator Del -command gat_deleteSelected
}

######################################################################
#
# Gate creation menus
#
proc tkg_makeMakeMenu {m} {
    menu $m
    $m add cascade -label "Switch" -menu $m.switch -underline 0
    $m add cascade -label "Gate" -menu $m.gate -underline 0
    $m add cascade -label "Reduction" -menu $m.reduction -underline 0
    $m add cascade -label "MSI" -menu $m.msi -underline 0
    $m add cascade -label "Alu" -menu $m.alu -underline 0
    $m add cascade -label "Memory" -menu $m.mem -underline 5
    $m add cascade -label "Module" -menu $m.block -underline 2
    $m add separator
    $m add command -label Comment -underline 0 -command "gat_make comment" -accelerator C
    $m add command -label Frame -underline 0 -command "gat_make frame" -accelerator F

    menu $m.switch -tearoff 0
    $m.switch add command -label Switch -underline 0 -command "gat_make switch" -accelerator s
    $m.switch add command -label "DIP Switch" -underline 0 -command "gat_make dip" -accelerator d
    $m.switch add command -label "Ground" -underline 0 -command "gat_make supply0" -accelerator g
    $m.switch add command -label "Vdd" -underline 0 -command "gat_make supply1" -accelerator v
    $m.switch add command -label "Wire Merge" -underline 0 -command "gat_make concat" -accelerator w
    $m.switch add separator
    $m.switch add command -label Clock -underline 0 -command "gat_make clock" -accelerator c
    $m.switch add command -label TTY -underline 0 -command "gat_make tty" -accelerator T

    menu $m.gate -tearoff 0
    $m.gate add command -label AND -underline 0 -accelerator a -command "gat_make and"
    $m.gate add command -label NAND -underline 3 -accelerator A -command "gat_make and -invert Z"
    $m.gate add command -label OR -underline 0 -accelerator o -command "gat_make or"
    $m.gate add command -label NOR -underline 2 -accelerator O -command "gat_make or -invert Z"
    $m.gate add command -label XOR -underline 0 -accelerator x -command "gat_make xor"
    $m.gate add command -label XNOR -underline 1 -accelerator X -command "gat_make xor -invert Z"
    $m.gate add command -label Buffer -underline 0 -accelerator b -command "gat_make buf"
    $m.gate add command -label Inverter -underline 0 -accelerator I -command "gat_make buf -invert Z"
    $m.gate add command -label Tri-Buffer -underline 0 -accelerator t -command "gat_make bufif1"
    $m.gate add separator
    $m.gate add command -label NMOS -underline 0 -accelerator "Ctl-t n" -command "gat_make nmos"
    $m.gate add command -label PMOS -underline 0 -accelerator "Ctl-t p"  -command "gat_make pmos"

    menu $m.reduction -tearoff 0
    $m.reduction add command -label AND -underline 0  -command "gat_make and -pins I=1" -accelerator "Ctl-r a"
    $m.reduction add command -label NAND -underline 3 -command "gat_make and -pins I=1 -invert Z" -accelerator "Ctl-r A"
    $m.reduction add command -label OR -underline 0  -command "gat_make or -pins I=1" -accelerator "Ctl-r o"
    $m.reduction add command -label NOR  -underline 2 -command "gat_make or -pins I=1 -invert Z" -accelerator "Ctl-r O"
    $m.reduction add command -label XOR -underline 0 -command "gat_make xor -pins I=1" -accelerator "Ctl-r x"
    $m.reduction add command -label XNOR -underline 1 -command "gat_make xor -pins I=1 -invert Z" -accelerator "Ctl-r X"

    menu $m.alu -tearoff 0
    $m.alu add command -label Adder -underline 0 -command "gat_make add" -accelerator +
    $m.alu add command -label Multiplier -underline 0 -command "gat_make mult" -accelerator "*"
    $m.alu add command -label Divider -underline 0 -command "gat_make div" -accelerator /
    $m.alu add separator
    $m.alu add command -label "Left Shift" -underline 0 -command "gat_make lshift" -accelerator SL
    $m.alu add command -label "Right Shift" -underline 0 -command "gat_make rshift" -accelerator SR
    $m.alu add command -label "Arith. Right Shift" -underline 0 -command "gat_make arshift" -accelerator SA
    $m.alu add command -label "Roll" -underline 0 -command "gat_make roll" -accelerator SO

    menu $m.mem -tearoff 0
    $m.mem add command -label Register -underline 0 -command "gat_make register" -accelerator r
    $m.mem add separator
    $m.mem add command -label RAM -underline 0 -command "gat_make ram" -accelerator R
    $m.mem add command -label ROM -underline 0 -command "gat_make rom" -accelerator u

    menu $m.msi -tearoff 0
    $m.msi add command -label "2-1 Multiplexor" -underline 0 -command "gat_make mux" -accelerator m
    $m.msi add command -label "4-1 Multiplexor" -underline 0 -command "gat_make mux -pins I=4" -accelerator "M 4"
    $m.msi add command -label "8-1 Multiplexor" -underline 0 -command "gat_make mux -pins I=8" -accelerator "M 8"
    $m.msi add separator
    $m.msi add command -label "1-2 Decoder" -underline 0 -command "gat_make demux" -accelerator "D 2"
    $m.msi add command -label "1-4 Decoder" -underline 0 -command "gat_make demux -pins Z=4" -accelerator "D 4"
    $m.msi add command -label "1-8 Decoder" -underline 0 -command "gat_make demux -pins Z=8" -accelerator "D 8"
    
    menu $m.block -tearoff 0
    $m.block add command -label "Module Instance" -underline 0 -accelerator B -command "gat_make block"
    $m.block add command -label "Module Input" -underline 7 -accelerator "\}" -command "gat_make input"
    $m.block add command -label "Module Output" -underline 7 -accelerator "\{" -command "gat_make output"
    $m.block add command -label "Module InOut" -underline 11 -accelerator "=" -command "gat_make inout"

}


menubutton .mbar.file -text File -underline 0 -menu .mbar.file.menu
tkg_makeFileMenu .mbar.file.menu

menubutton .mbar.edit -text Edit -underline 0 -menu .mbar.edit.menu 
tkg_makeEditMenu  .mbar.edit.menu

menubutton .mbar.tool -text Tool -underline 0 -menu .mbar.tool.menu 
tkg_makeToolMenu .mbar.tool.menu

menubutton .mbar.sim -text Simulate -underline 0 -menu .mbar.sim.menu
tkg_makeSimMenu .mbar.sim.menu normal

menubutton .mbar.module -text Module -underline 1 -menu .mbar.module.menu
tkg_makeModuleMenu  .mbar.module.menu


menubutton .mbar.gate -text Gate -underline 0 -menu .mbar.gate.menu
tkg_makeGateMenu  .mbar.gate.menu

menubutton .mbar.make -text Make -underline 0 -menu .mbar.make.menu
tkg_makeMakeMenu  .mbar.make.menu

######################################################################
#
# Help menu
#
menubutton .mbar.help -text Help -underline 0 -menu .mbar.help.menu
menu .mbar.help.menu
.mbar.help.menu add command -label "About..." -underline 0 -command showAbout
.mbar.help.menu add command -label "License..." -underline 0 -command showLicense
.mbar.help.menu add command -label "Documentation..." -underline 0 -command showDocumentation
.mbar.help.menu add separator
.mbar.help.menu add cascade -label "Tutorials" -menu .mbar.help.menu.tut -underline 1
.mbar.help.menu add cascade -label "Examples" -menu .mbar.help.menu.ex -underline 1

menu .mbar.help.menu.ex -tearoff 0
.mbar.help.menu.ex add command -label "Combinational Logic" -underline 0 -command "gat_load $tkg_gateHome/examples/ex1/combinational.v"
.mbar.help.menu.ex add command -label "8-Bit Counter" -underline 0 -command "gat_load $tkg_gateHome/examples/ex2/counter.v"
.mbar.help.menu.ex add command -label "Transistor-Level FF" -underline 0 -command "gat_load $tkg_gateHome/examples/ex3/trff.v"
.mbar.help.menu.ex add command -label "Menagerie CPU" -underline 12 -command "gat_load $tkg_gateHome/examples/ex4/menagerie.v"

menu .mbar.help.menu.tut -tearoff 0
.mbar.help.menu.tut add command -label "Creating a Circuit" -underline 0 -command "gat_load $tkg_gateHome/examples/tutorials/create_tut.v"
.mbar.help.menu.tut add command -label "Editing a Circuit (1)" -underline 19 -command "gat_load $tkg_gateHome/examples/tutorials/edit1_tut.v"
.mbar.help.menu.tut add command -label "Editing a Circuit (2)" -underline 19 -command "gat_load $tkg_gateHome/examples/tutorials/edit2_tut.v"
.mbar.help.menu.tut add command -label "Using Modules" -underline 0 -command "gat_load $tkg_gateHome/examples/tutorials/module_tut.v"
.mbar.help.menu.tut add command -label "Simulating a Circuit" -underline 0 -command "gat_load $tkg_gateHome/examples/tutorials/sim_tut.v"

######################################################################
#
# Final menu creation
#

pack .mbar.file .mbar.edit .mbar.tool  .mbar.sim .mbar.module .mbar.gate .mbar.make -in .mbar -side left
pack .mbar.help -in .mbar -side right
#tk_menuBar .mbar .mbar.file .mbar.edit .mbar.tool .mbar.sim .mbar.module .mbar.gate .mbar.make .mbar.help
#puts "disabled for linux"

