#   Copyright (C) 1987-1999 by Jeffery P. Hansen
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Last edit by hansen on Tue Jan  4 17:55:49 2000
#
# offsetgeometry win dx dy
#
# Return a geomtry string with the specified offset from the specified window.
#
#
proc offsetgeometry { win dx dy } {
  set g [wm geometry $win]
  set x [string range $g [expr [string first "+" $g] + 1] end]
  set p [string first "+" $x]
  set y [string range $x [expr $p + 1] end]
  set x [string range $x 0 [expr $p - 1]]
  return +[expr $x + $dx ]+[expr $y + $dy]
}

proc okcancel {w ok can} {
  frame $w -relief raised -bd 2
  button $w.ok -text OK -command $ok
  button $w.cancel -text Cancel -command $can
  pack $w.ok -side left -padx 5 -pady 5
  pack $w.cancel -side right -padx 5 -pady 5
}

#
# Insert an item i into listbox lb using comparison procedure cf.  The item
# is inserted at in sorted order, and it is assumed the listbox is
# already sorted.  A binary search is used for the insert.
#
proc lbSortInsert {lb iitem cf} {
  set b 0
  set e [$lb size]

  while { $b < $e } {
    set m [expr ($b + $e)/2 ]
    set litem [$lb get $m]
    set r [eval "$cf {$iitem} {$litem}"]
    if {$r < 0 } {
      set e [expr $m]
    } elseif {$r > 0 } {
      set b [expr $m + 1]
    } else {
      return
    }
  }
  $lb insert $b $iitem
}

#
# Same as lbSortInsert, except that an item it replaced if it already
# exists (decoration of the item may change).
#
proc lbSortRInsert {lb iitem cf} {
  set b 0
  set e [$lb size]

  while { $b < $e } {
    set m [expr ($b + $e)/2 ]
    set litem [$lb get $m]
    set r [eval "$cf {$iitem} {$litem}"]
    if {$r < 0 } {
      set e [expr $m]
    } elseif {$r > 0 } {
      set b [expr $m + 1]
    } else {
      $lb delete $m
      $lb insert $m $iitem
      return
    }
  }
  $lb insert $b $iitem
}

#
# Delete item iitem from listbox lb using comparison function cf.  It is
# assumed that the items in the listbox are sorted, as a binary search
# is used to find the item. 
#
proc lbSortDelete {lb iitem cf} {
  set b 0
  set e [$lb size]

  while { $b < $e } {
    set m [expr ($b + $e)/2 ]
    set litem [$lb get $m]
    set r [eval "$cf $iitem $litem"]
    if {$r < 0 } {
      set e [expr $m]
    } elseif {$r > 0} {
      set b [expr $m + 1]
    } else {
      $lb delete $m
      return
    }
  }
}

#
# Copy listbox contents of s to d.
#
proc lbCopy {s d} {
  set n [$s size]
  for {set i 0} { $i < $n } { incr i } {
    $d insert end [$s get $i]
  }
}

#
# Delete any occurances of chars from dset in str
#
proc tr-d {str dset} {
  set R ""
  set L [string length $str]

  for {set i 0} {$i < $L} {incr i} {
    set c [string index $str $i]
    if {[string first $c $dset] < 0} {
      append R $c
    }
  }

  return $R
}

#
# Replace the extension of name with ext.  "ext" should
# include the '.' character if it is desired.
#
proc replaceExtension {name ext} {
  set p [string last "." $name ]
  if { $p >= 0 } {
    set name [string range $name 0 [expr $p - 1]]
  }
  return "$name$ext"
}

#
# remove i from list l
#
proc ldelete {l i} {
  set p [lsearch $l $i]
  if { $p >= 0 } {
    return [lreplace $l $p $p]
  } {
    return $l
  }
}

bind HexEntry <Delete> { continue }
bind HexEntry <BackSpace> { continue }
bind HexEntry <Control-KeyPress> { continue }
bind HexEntry <KeyPress> {
  set c [string tolower %A]

  if { $c == "" } { continue }

  if { [string first $c "0123456789abcdef"] < 0 } { break }


  if {[%W selection present]} {
    %W delete sel.first sel.last
  } else {
    %W delete insert
  }
  set L [string length [%W get]]

  if { $L >= 8 } { break }

  tkEntryInsert %W $c

  break
}

proc tkg_setMode {n} {
  global mode simOn

  if { ! $simOn } {
    set mode $n
  }
}

proc copyVar {vd vs} {
  global $vd $vs
  eval "set $vd $$vs"
}

proc checkCopyVar {vd vs minv maxv} {
  global $vd $vs


  eval "set n $$vs"

  if { [scan $n "%d" n] != 1 } { set n $minv }

  if { $n < $minv } { set n $minv }
  if { $n > $maxv } { set n $maxv }

  eval "set $vd $n"
}

